@@ -5,10 +5,6 @@ This is the main execution file for OpenAccess_EPUB. It provides the primary
 mode of execution and interaction.
 """
 
-#If you change the version here, make sure to also change it in setup.py and
-#the module __init__.py
-__version__ = '0.3.5'
-
 #Standard Library Modules
 import argparse
 import sys
@@ -19,6 +15,7 @@ import traceback
 import multiprocessing
 
 #OpenAccess_EPUB Modules
+from ._version import __version__
 import openaccess_epub.utils as utils
 import openaccess_epub.utils.input as u_input
 from openaccess_epub.utils.images import get_images
@@ -72,9 +69,9 @@ def OAEParser():
     modes.add_argument('-p', '--parallel-batch', action='store', default=False,
                        help='''Use to specify a batch directory for parallel
                                processing.''')
-    #modes.add_argument('-C', '--collection', action='store', default=False,
-    #                   help='''Use to create an ePub file containing \
-    #                           multiple resources.''')
+    modes.add_argument('-C', '--collection', action='store_true', default=False,
+                       help='''Use to combine all xml files in the local
+                               directory into a single ePub collection.''')
     modes.add_argument('-cI', '--clear-image-cache', action='store_true',
                        default=False, help='''Clears the image cache''')
     modes.add_argument('-cC', '--clear-cache', action='store_true',
@@ -95,13 +92,15 @@ def dir_exists(outdirect):
     else:
         sys.exit('Aborting process!')
 
-def single_input(args, config):
+def single_input(args, config=None):
     """
     Single Input Mode works to convert a single input XML file into EPUB.
 
     This is probably the most typical use case and is the most highly
     configurable, see the argument parser and oaepub --help
     """
+    if config is None:
+        config = get_config_module()
     #Determination of input type and processing
     #Fetch by URL
     if 'http:' in args.input:
@@ -140,7 +139,7 @@ def single_input(args, config):
         epubcheck('{0}.epub'.format(output_name), config)
 
 
-def batch_input(args, config):
+def batch_input(args, config=None):
     """
     Batch Input Mode works to convert all of the article XML files in a
     specified directory into individual article EPUB files.
@@ -156,7 +155,8 @@ def batch_input(args, config):
     Batch Input Mode has default epubcheck behavior, it will place a system
     call to epubcheck unless specified otherwise (--no-epubcheck or -N flags).
     """
-    
+    if config is None:
+        config = get_config_module()
     error_file = open('batch_tracebacks.txt', 'w')
     #Iterate over all listed files in the batch directory
     for item in os.listdir(args.batch):
@@ -201,6 +201,104 @@ def batch_input(args, config):
     error_file.close()
 
 
+def collection_input(args, config=None):
+    """
+    Collection Input Mode works to compile multiple articles into a single
+    composite ePub. This is akin to such formats as Collections, Issues, and
+    Omnibus; it may also be useful for those interested in the simple
+    distribution of a reading list, personal publications, or topic reference.
+
+    Collection Input Mode produces output that is necessarily unlike the output
+    generated by Single or Batch (which is just sequential Single) input modes.
+    The primary difference in output lies with the ePub metadata; as applying
+    metadata from any single article to the whole would be inappropriate.
+
+    Unlike other input modes, Collection Mode is strictly dependent on the
+    local directory of execution. If there is a file named "order.txt" in the
+    local directory, this file should contain the name of one input XML file
+    on each line; the files will be added to the ePub output by line-order.
+    If there is "order.txt" file, Collection Mode will assume that all XML
+    files are input and the article in order in the collection will be random.
+
+    Collection Input Mode has default epubcheck behavior, it will place a system
+    call to epubcheck unless specified otherwise (--no-epubcheck or -N flags).
+    """
+    if config is None:
+        config = get_config_module()
+    try:
+        order = open('order.txt', 'r')
+    except IOError:  # No order.txt
+        xml_files = list_xml_files(dir=os.getcwd())
+    else:
+        #Add all nonempty lines, in order, to the xml_files list
+        xml_files = [i.strip() for i in order.readlines() if i.strip()]
+        order.close()
+
+    #The output name will be the same as the parent directory name
+    #This will also serve as the dc:title
+    output_name = os.path.split(os.getcwd())[1]
+
+    #The standard make_epub() method will not work for Collection Mode
+    #So the work done here is an adaptation of it
+    print('Processing output to {0}.epub'.format(output_name))
+    #Copy files from base_epub to the new output
+    if os.path.isdir(output_name):
+        dir_exists(output_name)
+    epub_base = os.path.join(CACHE_LOCATION, 'base_epub')
+    shutil.copytree(epub_base, output_name)
+    
+    toc = ncx.TocNCX(version=__version__, collection_mode=True)
+    myopf = opf.PLoSOPF(__version__, output_name, collection_mode=True, title=output_name)
+    
+    #Now it is time to operate on each of the xml files
+    for xml_file in xml_files:
+        raw_name = u_input.local_input(xml_file)  # is this used?
+        parsed_article = Article(xml_file)
+    
+        if parsed_article.metadata.dtdVersion() == '2.0':  #Not supported
+            print('Article published with JPTS DTDv2.0, not supported!')
+            sys.exit(1)
+        #Get the Digital Object Identifier
+        doi = parsed_article.getDOI()
+        journal_doi, article_doi = doi.split('/')
+        
+        #Check for images
+        img_dir = os.path.join(output_name, 'OPS', 'images-{0}'.format(article_doi))
+        expected_local = 'images-{0}'.format(raw_name)
+        if os.path.isdir(expected_local):
+            utils.images.local_images(expected_local, img_dir)
+        else:
+            article_cache = os.path.join(config.image_cache, journal_doi, article_doi)
+            if os.path.isdir(article_cache):
+                utils.images.image_cache(article_cache, img_dir)
+            else:
+                print('Images for {0} (DOI: {1}) could not be found!'.format(xml_file, doi))
+                r = input('Try to download them? [Y/n]')
+                if r in ['y', 'Y', '']:
+                    os.mkdir(img_dir)
+                    utils.images.fetch_plos_images(article_doi, img_dir, parsed_article)
+                    if config.use_image_cache:
+                        utils.images.move_images_to_cache(img_dir, article_cache)
+                else:
+                    sys.exit(1)
+
+        #TODO: Content stuff
+        if journal_doi == '10.1371':  # PLoS's publisher DOI
+            ops_doc = ops.OPSPLoS(parsed_article, output_name)
+            #TODO: Workflow change, parse table of contents from OPS processed document
+            toc.parse_article(parsed_article)
+            myopf.parse_article(parsed_article)
+    toc.write(output_name)
+    myopf.write()
+    utils.epubZip(output_name)
+    
+
+    #Running epubcheck on the output verifies the validity of the ePub,
+    #requires a local installation of java and epubcheck.
+    if args.no_epubcheck:
+        epubcheck('{0}.epub'.format(output_name), config)
+
+
 class ParallelBatchProcess(multiprocessing.Process):
     """
     
@@ -284,27 +382,17 @@ def parallel_batch_input(args, config):
     tasks.join()
     error_file.close()
 
-def collection_input(args, config):
-    """
-    Collection Input Mode is intended for the combination of multiple articles
-    into a single ePub file. This may be useful for producing "Volumes", custom
-    reading lists for classroom use, and compendia on common subjects.
 
-    There is a lot of potential for how this might be used, development will
-    proceed in the direction of interest.
-    """
-    pass
-
-
-def zipped_input(args, config):
+def zipped_input(args, config=None):
     """
     Zipped Input Mode is primarily intended as a workflow for Frontiers
     articles, where the article xml and relevant images are zipped together.
     """
-    pass
+    if config is None:
+        config = get_config_module()
 
 
-def make_epub(document, outdirect, explicit_images, batch, config):
+def make_epub(document, outdirect, explicit_images, batch, config=None):
     """
     Encapsulates the primary processing work-flow. Before this method is
     called, pre-processing has occurred to define important directory and file
@@ -312,7 +400,8 @@ def make_epub(document, outdirect, explicit_images, batch, config):
     to generate the ePub content.
     """
     print('Processing output to {0}.epub'.format(outdirect))
-
+    if config is None:
+        config = get_config_module()
     #Copy files from base_epub to the new output
     if os.path.isdir(outdirect):
         if batch:
@@ -333,16 +422,16 @@ def make_epub(document, outdirect, explicit_images, batch, config):
 
     #Run content processing per publisher
     if DOI.split('/')[0] == '10.1371':  # PLoS's publisher DOI
-        ops.OPSPLoS(document, outdirect)
+        ops_doc = ops.OPSPLoS(document, outdirect)
         #TODO: Workflow change, parse table of contents from OPS processed document
         toc = ncx.TocNCX(__version__)
-        toc.parseArticle(document)
+        toc.parse_article(document)
         toc.write(outdirect)
         myopf = opf.PLoSOPF(__version__, outdirect, False)
     elif DOI.split('/')[0] == '10.3389':  # Frontiers' publisher DOI
         ops.OPSFrontiers(document, outdirect)
         toc = ncx.TocNCX(__version__)
-        toc.parseArticle(document)
+        toc.parse_article(document)
         toc.write(outdirect)
         myopf = opf.FrontiersOPF(__version__, outdirect, False)
     myopf.parse_article(document)
@@ -350,12 +439,14 @@ def make_epub(document, outdirect, explicit_images, batch, config):
     utils.epubZip(outdirect)
 
 
-def epubcheck(epubname, config):
+def epubcheck(epubname, config=None):
     """
     This method takes the name of an epub file as an argument. This name is
     the input for the java execution of a locally installed epubcheck-.jar. The
     location of this .jar file is configured in config.py.
     """
+    if config is None:
+        config = get_config_module()
     r, e = os.path.splitext(epubname)
     if not e:
         print('Warning: Filename extension is empty, appending \'.epub\'...')
@@ -378,10 +469,25 @@ def get_config_module():
         config = imp.load_source('config', config_path)
     except IOError:
         print('Could not find {0}, please run oae-quickstart'.format(config_path))
-        sys.exit()
+        sys.exit(1)
     else:
         return config
 
+
+def list_xml_files(dir):
+    """
+    Receives a path to a system directory and then returns a list of all XML
+    files contained in the directory.
+    """
+    xml_files = []
+    for item in os.listdir(dir):
+        item_path = os.path.join(dir, item)
+        #Skip directories and files without .xml extension
+        _root, extension = os.path.splitext(item_path)
+        if os.path.isfile(item_path) and extension == '.xml':
+            xml_files.append(item_path)
+    return xml_files
+
 def main(args):
     """
     This is the main code execution block.
