@@ -10,24 +10,25 @@ conversion.
 
 import openaccess_epub.dublincore as dc
 import openaccess_epub.utils as utils
-import datetime
-import os.path
+import uuid
+import time
+import os
 import xml.dom.minidom
 import logging
 
 log = logging.getLogger('OPF')
 
-
 class MetaOPF(object):
     """
     Represents the OPF document and the methods needed to produce it. Dublin
     Core metadata is referenced by this class per publisher.
     """
 
-    def __init__(self, version, location, collection_mode=False):
+    def __init__(self, version, location, collection_mode=False, title=None):
         log.info('Instantiating OPF class')
         self.doi = ''
         self.dois = []
+        self.title = title
         self.collection_mode = collection_mode
         if self.collection_mode:
             log.debug('Collection Mode')
@@ -60,11 +61,6 @@ class MetaOPF(object):
             self.package.appendChild(self.doc.createElement(el))
         self.metadata, self.manifest, self.spine, self.guide = self.package.childNodes
         self.spine.setAttribute('toc', 'ncx')
-        #Here we create a custom collection unique identifier string
-        #Consists of software name and version along with timestamp
-        t = datetime.datetime(1, 1, 1)
-        self.ccuid = 'OpenAccess_EPUBv{0}-{1}'.format('__version__',
-                                                      t.utcnow().__str__())
 
     def parse_article(self, article):
         """
@@ -149,8 +145,24 @@ class MetaOPF(object):
         if self.article.root_tag.getElementsByTagName('table-wrap'):
             self.spine.appendChild(tab_ref)
 
+    def finalize_metadata(self):
+        """Do any final metadata operations before writing."""
+        if self.collection_mode:  #The primary use case for finalization
+            #Make the dc:identifier as UUID
+            dc_identifier = dc.identifier(str(uuid.uuid4()), self.doc, primary=True)
+            dc_identifier.setAttribute('opf:scheme', 'UUID')
+            self.metadata.appendChild(dc_identifier)
+            #Make the dc:language as english
+            self.metadata.appendChild(dc.language(self.doc))  # English default
+            #Make the dc:title using the passed title argument
+            #TODO: Make titling for collections... better
+            self.metadata.appendChild(dc.title(self.title, self.doc))
+        else:
+            pass
+
     def write(self):
         self.make_manifest()
+        self.finalize_metadata()
         filename = os.path.join(self.location, 'OPS', 'content.opf')
         with open(filename, 'wb') as output:
             output.write(self.doc.toprettyxml(encoding='utf-8'))
@@ -364,6 +376,7 @@ class PLoSOPF(MetaOPF):
         #Create the dc:relation for related articles
         #This is not yet implemented
         #self.metadata.appendChild(dc.relation(relation, self.doc))
+        #TODO: Implement this?
 
         #Create the dc:publisher element
         self.metadata.appendChild(dc.publisher('Public Library of Science', self.doc))
@@ -386,6 +399,9 @@ class PLoSOPF(MetaOPF):
         This method handles the metadata for a PLoS article in a collection.
         """
         log.info('Using PLoS collectionMetadata')
+        #For brevity
+        ameta = article_metadata
+        
 
     def add_to_spine(self):
         idref = '{0}-' + '{0}-xml'.format(self.a_doi_dashed)
