@@ -29,7 +29,7 @@ class OPSPLoS(OPSMeta):
         self.make_fragment_identifiers()
         self.ops_dir = os.path.join(output_dir, 'OPS')
         self.html_tables = []
-        self.create_main()
+        self.main_body = self.create_main()
         self.create_biblio()
         if self.html_tables:
             self.create_tables()
@@ -103,6 +103,7 @@ class OPSPLoS(OPSMeta):
         with open(os.path.join(self.ops_dir, self.main_frag[:-4]), 'wb') as op:
             op.write(self.document.toxml(encoding='utf-8'))
             #op.write(self.document.toprettyxml(encoding='utf-8'))
+        return body
 
     def make_heading(self, receiving_node):
         """
@@ -185,8 +186,8 @@ class OPSPLoS(OPSMeta):
         This method encapsulates the functions necessary to create the biblio
         segment of the article.
         """
-        self.doc = self.make_document('biblio')
-        body = self.doc.getElementsByTagName('body')[0]
+        self.document = self.make_document('biblio')
+        body = self.document.getElementsByTagName('body')[0]
         body.setAttribute('id', 'references')
         try:
             back = self.article.getElementsByTagName('back')[0]
@@ -203,8 +204,8 @@ class OPSPLoS(OPSMeta):
 
         #Finally, write to a document
         with open(os.path.join(self.ops_dir, self.bib_frag[:-4]), 'wb') as op:
-            op.write(self.doc.toxml(encoding='utf-8'))
-            #op.write(self.doc.toprettyxml(encoding='utf-8'))
+            op.write(self.document.toxml(encoding='utf-8'))
+            #op.write(self.document.toprettyxml(encoding='utf-8'))
 
     def create_tables(self):
         """
@@ -344,13 +345,17 @@ class OPSPLoS(OPSMeta):
             #<label> might be missing, especially for one author works
             label_node = aff.getChildrenByTagName('label')
             #I expect there to always be the <addr-line>
-            addr_line_node = aff.getChildrenByTagName('addr-line')[0]
+            addr_line_node = aff.getOptionalChild('addr-line')
+            if addr_line_node:
+                addr_line_text = utils.nodeText(addr_line_node)
+            else:
+                addr_line_text = utils.getTagText(aff)
             cur_aff_span = self.appendNewElement('span', affs_div)
             cur_aff_span.setAttribute('id', aff_id)
             if label_node:
                 label_text = utils.nodeText(label_node[0])
                 label_b = self.appendNewElementWithText('b', label_text, cur_aff_span)
-            addr_line_text = utils.nodeText(addr_line_node)
+            
             self.appendNewText(addr_line_text + ', ', cur_aff_span)
 
     def make_heading_abstracts(self, receiving_node):
@@ -767,9 +772,9 @@ class OPSPLoS(OPSMeta):
             #Create OPS content, using image path, label, and caption
             fig_parent = fig.parentNode
             #Create a horizontal rule
-            fig_parent.insertBefore(self.doc.createElement('hr'), fig)
+            fig_parent.insertBefore(self.document.createElement('hr'), fig)
             #Create the img element
-            img_element = self.doc.createElement('img')
+            img_element = self.document.createElement('img')
             img_element.setAttribute('alt', 'A Figure')
             if 'id' in fig_attributes:
                 img_element.setAttribute('id', fig_attributes['id'])
@@ -780,7 +785,7 @@ class OPSPLoS(OPSMeta):
 
             #Create content for the label and caption
             if caption_node or label_node:  # These will go into a <div> after <img>
-                img_caption_div = self.doc.createElement('div')
+                img_caption_div = self.document.createElement('div')
                 img_caption_div.setAttribute('class', 'figure-caption')
                 img_caption_div_b = self.appendNewElement('b', img_caption_div)
                 if label_node:
@@ -801,7 +806,7 @@ class OPSPLoS(OPSMeta):
                 fig_parent.insertBefore(img_caption_div, fig)
 
             #Create a horizontal rule
-            fig_parent.insertBefore(self.doc.createElement('hr'), fig)
+            fig_parent.insertBefore(self.document.createElement('hr'), fig)
 
             #Remove the original <fig>
             fig.removeSelf()
@@ -830,20 +835,28 @@ class OPSPLoS(OPSMeta):
             except IndexError:
                 caption_node = None
 
-            #Get the alternatives node, for some articles it will hold the graphic
-            alternatives = tab.getChildrenByTagName('alternatives')
-            if alternatives:  # New articles are this way
-                graphic_node = alternatives[0].getChildrenByTagName('graphic')[0]
-                table_node = alternatives[0].getChildrenByTagName('table')[0]
-            elif tab.getChildrenByTagName('graphic'): # Old articles seem to be this way
-                #TODO: Find some way to properly render out the HTML comments
-                graphic_node = tab.getChildrenByTagName('graphic')[0]
-                table_node = False
-            else:  # Article with neither graphic nor alternative, must have table
-                table = tab.getChildrenByTagName('table')[0]
-                tab.replaceSelfWith(table)
-                continue
-                
+            #Get the alternatives node, for almost all articles, it will hold
+            #the <graphic> and the <table>
+            #I am assuming only one graphic and only one table per table-wrap
+            alternatives = tab.getOptionalChild('alternatives')
+            if alternatives:
+                graphic_node = alternatives.getOptionalChild('graphic')
+                table_node = alternatives.getOptionalChild('table')
+            #If the article doesn't have the <alternatives> node, or either of
+            #these are not under alternatives, check directly under table-wrap
+            else:
+                graphic_node = None
+                table_node = None
+            if not graphic_node:
+                graphic_node = tab.getOptionalChild('graphic')
+            if not table_node:
+                table_node = tab.getOptionalChild('table')
+
+            if not graphic_node and table_node:
+                tab.replaceSelfWith(table_node)
+                continue  # Just replace table-wrap with this node and move on
+            #Past this point, there must be a graphic... fails if neither
+
             #Label and move the html table node to the list of html tables
             if table_node:
                 if 'id' in tab_attributes:
@@ -866,9 +879,9 @@ class OPSPLoS(OPSMeta):
             #Create OPS content, using image path, label, and caption
             tab_parent = tab.parentNode
             #Create a horizontal rule
-            tab_parent.insertBefore(self.doc.createElement('hr'), tab)
+            tab_parent.insertBefore(self.document.createElement('hr'), tab)
             #Create the img element
-            img_element = self.doc.createElement('img')
+            img_element = self.document.createElement('img')
             img_element.setAttribute('alt', 'A Table')
             if 'id' in tab_attributes:
                 img_element.setAttribute('id', tab_attributes['id'])
@@ -877,7 +890,7 @@ class OPSPLoS(OPSMeta):
 
             #Create content for the label and caption
             if caption_node or label_node:  # These will go into a <div> before <img>
-                img_caption_div = self.doc.createElement('div')
+                img_caption_div = self.document.createElement('div')
                 img_caption_div.setAttribute('class', 'table-caption')
                 img_caption_div_b = self.appendNewElement('b', img_caption_div)
                 if label_node:
@@ -902,13 +915,13 @@ class OPSPLoS(OPSMeta):
 
             #Create a link to the html version of the table
             if table_node:
-                html_table_link = self.doc.createElement('a')
+                html_table_link = self.document.createElement('a')
                 html_table_link.setAttribute('href', self.tab_frag.format(tab_attributes['id']))
                 self.appendNewText('HTML version of this table', html_table_link)
                 tab_parent.insertBefore(html_table_link, tab)
 
             #Create a horizontal rule
-            tab_parent.insertBefore(self.doc.createElement('hr'), tab)
+            tab_parent.insertBefore(self.document.createElement('hr'), tab)
 
             #Remove the original <table-wrap>
             tab_parent.removeChild(tab)
@@ -956,9 +969,13 @@ class OPSPLoS(OPSMeta):
                 if not div_title.childNodes:
                     div.removeChild(div_title)
                 else:
-                    div_title.tagName = depth_tags[depth]
+                    if depth < len(depth_tags):
+                        div_title.tagName = depth_tags[depth]
+                    else:
+                        div_title.tagName = 'span'
+                        div_title.setAttribute('class', 'extendedheader{0}'.format(depth+2))
                     if div_label_text:
-                        label_node = self.doc.createTextNode(div_label_text)
+                        label_node = self.document.createTextNode(div_label_text)
                         div_title.insertBefore(label_node, div_title.firstChild)
             self.convert_div_titles(div, depth=depth + 1)
 
@@ -1009,7 +1026,28 @@ class OPSPLoS(OPSMeta):
             except IndexError:  # No label tag
                 label_node = None
 
-            #Get the graphic node in the fig, treat as mandatory
+            #Get the graphic node in the disp, not always present
+            graphic_node = disp.getOptionalChild('graphic')
+            #If graphic not present
+            if not graphic_node:  #Assume there is math text instead
+                text_span = self.document.createElement('span')
+                if 'id' in disp_attributes:
+                    text_span.setAttribute('id', disp_attributes['id'])
+                text_span.setAttribute('class', 'disp-formula')
+                text_span.childNodes = disp.childNodes
+                #Create OPS content, using image path, and label
+                disp_parent = disp.parentNode
+                #Insert the img element
+                disp_parent.insertBefore(text_span, disp)
+                #Create content for the label
+                if label_node:
+                    disp_parent.insertBefore(label_node, text_span)
+                    label_node.tagName = 'b'
+                #Remove the old disp-formula element
+                disp.removeSelf()
+                continue
+            
+            #If graphic present
             graphic_node = disp.getChildrenByTagName('graphic')[0]
             #Create a file reference for the image
             graphic_xlink_href = graphic_node.getAttribute('xlink:href')
@@ -1021,7 +1059,7 @@ class OPSPLoS(OPSMeta):
             disp_parent = disp.parentNode
 
             #Create the img element
-            img_element = self.doc.createElement('img')
+            img_element = self.document.createElement('img')
             img_element.setAttribute('alt', 'A Display Formula')
             if 'id' in disp_attributes:
                 img_element.setAttribute('id', disp_attributes['id'])
@@ -1092,41 +1130,12 @@ class OPSPLoS(OPSMeta):
         Extract or extended quoted passage from another work, usually made
         typographically distinct from surrounding text
 
-        <disp-quote> elements have a relatively complex content model, but it
-        appears that PLoS typically employs a simple <p> child element to hold
-        all of the text, until otherwise found, this method handles the
-        conversion under this assumption.
+        <disp-quote> elements have a relatively complex content model, but PLoS
+        appears to employ either <p>s or <list>s.
         """
         for disp_quote in body.getElementsByTagName('disp-quote'):
-            disp_quote_parent = disp_quote.parentNode
-            if disp_quote_parent.tagName == 'p':
-                grandparent = disp_quote_parent.parentNode
-                disp_quote.elevateNode()
-                if not disp_quote_parent.childNodes:
-                    grandparent.removeChild(disp_quote_parent)
-                #The grandparent is now the new parent
-                disp_quote_parent = grandparent
-            top_hr = self.doc.createElement('hr')
-            bottom_hr = self.doc.createElement('hr')
-            #Most <disp-quote>s contain only a single <p>
-            #But journal.pone.0002246 shows a list instead, assume the general
-            #case of one or the other form
-            if disp_quote.getChildrenByTagName('list'):
-                list = disp_quote.getChildrenByTagName('list')[0]
-                disp_quote_parent.insertBefore(top_hr, disp_quote)
-                top_hr.setAttribute('class', 'disp-quote')
-                for list_item in list.getChildrenByTagName('list-item'):
-                    list_p = list_item.getChildrenByTagName('p')[0]
-                    list_p.setAttribute('class', 'disp-quote')
-                    disp_quote_parent.insertBefore(list_p, disp_quote)
-                disp_quote_parent.insertBefore(bottom_hr, disp_quote)
-                bottom_hr.setAttribute('class', 'disp-quote')
-            else:
-                paragraph = disp_quote.getChildrenByTagName('p')[0]
-                for element in [top_hr, paragraph, bottom_hr]:
-                    element.setAttribute('class', 'disp-quote')
-                    disp_quote_parent.insertBefore(element, disp_quote)
-            disp_quote_parent.removeChild(disp_quote)
+            disp_quote.tagName = 'div'
+            disp_quote.setAttribute('class', 'disp-quote')
 
     def convert_boxed_text_elements(self, body):
         """
@@ -1142,18 +1151,14 @@ class OPSPLoS(OPSMeta):
         for boxed_text in body.getElementsByTagName('boxed-text'):
             boxed_text_attrs = boxed_text.getAllAttributes(remove=False)
             boxed_text_parent = boxed_text.parentNode
-            top_hr = self.doc.createElement('hr')
-            bottom_hr = self.doc.createElement('hr')
             sec = boxed_text.getOptionalChild('sec')
-            #sec = boxed_text.getChildrenByTagName('sec')[0]
             if sec:
                 sec.tagName = 'div'
                 title = sec.getOptionalChild('title')
-                #title = sec.getChildrenByTagName('title')
                 if title:
                     title.tagName = 'b'
             else:
-                sec = self.doc.createElement('div')
+                sec = self.document.createElement('div')
                 sec.childNodes = boxed_text.childNodes
             sec.setAttribute('class', 'boxed-text')
             if 'id' in boxed_text_attrs:
@@ -1231,7 +1236,7 @@ class OPSPLoS(OPSMeta):
             verse_group.tagName = 'div'
             verse_group.setAttribute('id', 'verse-group')
             if any([label, title, subtitle]):
-                new_verse_title = self.doc.createElement('b')
+                new_verse_title = self.document.createElement('b')
                 verse_group.insertBefore(verse_group.firstChild)
                 if label:
                     new_verse_title.childNodes += label[0].childNodes
@@ -1336,7 +1341,10 @@ class OPSPLoS(OPSMeta):
         for list_el in body.getElementsByTagName('list'):
             list_el_attributes = list_el.getAllAttributes(remove=True)
             list_el_parent = list_el.parentNode
-            list_el_type = list_el_attributes['list-type']
+            try:
+                list_el_type = list_el_attributes['list-type']
+            except KeyError:
+                list_el_type = 'order'
             #Unordered list if '', 'bullet', or 'simple'
             if list_el_type in ['', 'bullet', 'simple']:
                 list_el.tagName = 'ul'
